package picturelab;

import java.awt.*;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.text.*;
import java.util.*;
import java.util.List; // resolves problem with java.awt.List and java.util.List

/**
 * A class that represents a picture. This class inherits from SimplePicture and
 * allows the student to add functionality to the Picture class.
 *
 * @author Barbara Ericson ericson@cc.gatech.edu
 */
public class Picture extends SimplePicture {
  ///////////////////// constructors //////////////////////////////////

   /**
    * Constructor that takes no arguments
    */
   public Picture() {
      /* not needed but use it to show students the implicit call to super()
       * child constructors always call a parent constructor 
       */
      super();
   }

   /**
    * Constructor that takes a file name and creates the picture
    *
    * @param fileName the name of the file to create the picture from
    */
   public Picture(String fileName) {
      // let the parent class handle this fileName
      super(fileName);
   }

   /**
    * Constructor that takes a file name and creates the picture
    *
    * @param file the name of the file to create the picture from
    */
   public Picture(java.io.File file) {
      // let the parent class handle this fileName
      super(file);
   }
   
   /**
    * Constructor that takes the width and height
    * 
    * @param height the height of the desired picture
    * @param width the width of the desired picture
    */
   public Picture(int width, int height) {
      // let the parent class handle this width and height
      super(width, height);
   }

   /**
    * Constructor that takes a picture and creates a copy of that picture
    *
    * @param copyPicture the picture to copy
    */
   public Picture(Picture copyPicture) {
      // let the parent class do the copy
      super(copyPicture);
   }

   /**
    * Constructor that takes a buffered image
    *
    * @param image the buffered image to use
    */
   public Picture(BufferedImage image) {
      super(image);
   }

  ////////////////////// methods ///////////////////////////////////////
   /**
    * Method to return a string with information about this picture.
    *
    * @return a string with information about the picture such as fileName,
    * height and width.
    */
   public String toString() {
      String output = "Picture, filename " + getFileName()
              + " height " + getHeight()
              + " width " + getWidth();
      return output;

   }

   /**
    * Method to set the blue to 0
    */
   public void zeroBlue() {
      Pixel[][] pixels = this.getPixels2D();
      for (Pixel[] rowArray : pixels) {
         for (Pixel pixelObj : rowArray) {
            pixelObj.setBlue(0);
         }
      }
   }
   
   public void zeroRed() {
      Pixel[][] pixels = this.getPixels2D();
      for(int r=0; r<pixels.length; r++) {
         for(int c=0; c<pixels[0].length; c++)
            pixels[r][c].setRed(0);
      }
   }

   public void zeroGreen() {
      Pixel[][] pixels = this.getPixels2D();
      for(int r=0; r<pixels.length; r++) {
         for(int c=0; c<pixels[0].length; c++)
            pixels[r][c].setGreen(0);
      }
   }

   /**
    * Method that mirrors the picture around a vertical mirror in the center of
    * the picture from left to right
    */
   public void mirrorVertical() {
      Pixel[][] pixels = this.getPixels2D();
      Pixel leftPixel = null;
      Pixel rightPixel = null;
      int width = pixels[0].length;
      for (int row = 0; row < pixels.length; row++) {
         for (int col = 0; col < width / 2; col++) {
            leftPixel = pixels[row][col];
            rightPixel = pixels[row][width - 1 - col];
            rightPixel.setColor(leftPixel.getColor());
         }
      }
   }
   
   /**
    *  Creates a vertical mirror image of the this picture.
    */
   public void verticalReflection() {
      Pixel[][] pixels = this.getPixels2D();
      Pixel leftPixel,rightPixel;
      int width = pixels[0].length;
      for (int row = 0; row < pixels.length; row++) {
         for (int col = 0; col < width / 2; col++) {
            leftPixel = pixels[row][col];
            rightPixel = pixels[row][width - 1 - col];
            rightPixel.setColor(leftPixel.getColor());
         }
      }
      
      
   }
   
   /**
    * Converts this image into a horizontal mirror image of itself.
    */
   public void horizontalReflection() {
      for(int y=0; y<getHeight()/2; y++) {      
         for(int x=0; x<getWidth(); x++) {
            int color1 = getBasicPixel(x, y);
            int color2 = getBasicPixel(x, getHeight() - y-1);
            setBasicPixel(x, y, color2);
            setBasicPixel(x, getHeight() - y-1, color1);
         }
      }
   }

   /**
    * Mirror just part of a picture of a temple
    */
   public void mirrorTemple() {
      int mirrorPoint = 276;
      Pixel leftPixel = null;
      Pixel rightPixel = null;
      int count = 0;
      Pixel[][] pixels = this.getPixels2D();

      // loop through the rows
      for (int row = 27; row < 97; row++) {
         // loop from 13 to just before the mirror point
         for (int col = 13; col < mirrorPoint; col++) {

            leftPixel = pixels[row][col];
            rightPixel = pixels[row][mirrorPoint - col + mirrorPoint];
            rightPixel.setColor(leftPixel.getColor());
         }
      }
   }

   /**
    * copy from the passed fromPic to the specified startRow and startCol in the
    * current picture
    *
    * @param fromPic the picture to copy from
    * @param startRow the start row to copy to
    * @param startCol the start col to copy to
    */
   public void copy(Picture fromPic,
           int startRow, int startCol) {
      Pixel fromPixel = null;
      Pixel toPixel = null;
      Pixel[][] toPixels = this.getPixels2D();
      Pixel[][] fromPixels = fromPic.getPixels2D();
      for (int fromRow = 0, toRow = startRow;
              fromRow < fromPixels.length
              && toRow < toPixels.length;
              fromRow++, toRow++) {
         for (int fromCol = 0, toCol = startCol;
                 fromCol < fromPixels[0].length
                 && toCol < toPixels[0].length;
                 fromCol++, toCol++) {
            fromPixel = fromPixels[fromRow][fromCol];
            toPixel = toPixels[toRow][toCol];
            toPixel.setColor(fromPixel.getColor());
         }
      }
   }

/**
 * 
 * @param fromPic The source image we are copying from
 * @param destRow the start row to copy to
 * @param destCol the start col to copy to
 * @param fromRow The start row of fromPic
 * @param fromCol The start col of fromPic
 * @param w Width of the area we wish to copy.
 * @param h Height of the area we wish to copy.
 */
   public void copy(Picture fromPic,int destRow, int destCol, int fromRow, int fromCol, int w, int h) {
      //TODO: Write and test this method
   }
 
   
   /**
    * Method to show large changes in color
    *
    * @param edgeDist the distance for finding edges
    */
   public void edgeDetection(int edgeDist) {
      Pixel leftPixel = null;
      Pixel rightPixel = null;
      Pixel[][] pixels = this.getPixels2D();
      Color rightColor = null;
      for (int row = 0; row < pixels.length; row++) {
         for (int col = 0;
                 col < pixels[0].length - 1; col++) {
            leftPixel = pixels[row][col];
            rightPixel = pixels[row][col + 1];
            rightColor = rightPixel.getColor();
            if (leftPixel.colorDistance(rightColor)
                    > edgeDist)
               leftPixel.setColor(Color.BLACK);
            else
               leftPixel.setColor(Color.WHITE);
         }
      }
   }

   /* Main method for testing - each class in Java can have a main 
    * method 
    */
   public static void main(String[] args) {
      Picture beach = new Picture("beach.jpg");
      beach.explore();
      beach.zeroBlue();
      beach.explore();
   }

} // this } is the end of class Picture, put all new methods before this
