package picturelab;

import java.awt.Color;
/**
 *
 * @author LaSpina, Bethpage High School
 */

public class PictureUtils {

   /**
    * Reduces the picture to the given dimensions
    * @param scaleMe The original image to be resized.
    * @param width The desired width of the output image.
    * @param height The desired height of the output image
    * @return A new image that has been scaled to the given dimensions.
    */
   static Picture scaleTo(Picture scaleMe, int width, int height) {
      double originalWidth = scaleMe.getWidth();
      double originalHeight = scaleMe.getHeight();
      return scaleMe.scale(width / originalWidth, height / originalHeight);
   }
   
   /**
    * Used to resize an image where the width and height are changed by the same scale factor.
    * @param scaleMe The original image to be resized.
    * @param width The desired width of the output image.
    * @return A new image that has been scaled to the given width, where the height has
    *          been adjusted by the same ratio.
    */
   static Picture scaleToKeepAspectratio(Picture scaleMe, int width) {
      double ratio = 1.0 * width / scaleMe.getWidth();
      return scaleMe.scale(ratio, ratio);
   }
   
   /**
    * Computes the average color
    * @param p
    * @return 
    */
   static int averageBrightness(Picture p) {
      Color avg = averageColor(p);
      double avgRGB = (avg.getRed() + avg.getGreen() + avg.getBlue())/3.0;
      return (int) Math.round(avgRGB);
   }
   
   /**
    * Computes the average red, green, and blue values of all pixels in the given image.
    * @param image The source picture to analyze.
    * @return An "average color" created by instantiating a color object with the average values of red, green and blue.
    */
   static Color averageColor(Picture image) {
      Pixel[] allPixels = image.getPixels();
      int redSum, blueSum, greenSum;
      redSum = blueSum = greenSum = 0;
      for(Pixel p : allPixels) {
         redSum += p.getRed();
         blueSum += p.getBlue();
         greenSum += p.getGreen();
      }
      double redAvg = Math.round(1.0*redSum / allPixels.length);
      double greenAvg = Math.round(1.0*greenSum / allPixels.length);
      double blueAvg = Math.round(1.0*blueSum / allPixels.length);
      return new Color( (int)redAvg, (int)blueAvg, (int)greenAvg );
   }
   
   /**
    * Averages two images together.
    * @param image1
    * @param image2
    * @return 
    */
   static Picture blend(Picture image1, Picture image2) {
      if(image1.getHeight()!=image2.getHeight() ||
         image1.getWidth() != image2.getWidth())
         throw new IllegalArgumentException("Dimension mismatch error: Cannot blend two images with different sizes.");
      Picture blendedImage = new Picture(image1.getWidth(), image1.getHeight());
      Pixel[][] pixels1 = image1.getPixels2D();
      Pixel[][] pixels2 = image2.getPixels2D();
      Pixel[][] blendPixels = blendedImage.getPixels2D();
      for(int r=0; r<pixels1.length; r++) {
         for(int c=0; c<pixels1[0].length; c++) {
            int red = (pixels1[r][c].getRed() + pixels2[r][c].getRed())/2;
            int green = (pixels1[r][c].getGreen() + pixels2[r][c].getGreen())/2;
            int blue = (pixels1[r][c].getBlue() + pixels2[r][c].getBlue())/2;
            blendPixels[r][c].setColor(new Color(red,green,blue));
         }
      }
      blendedImage.setTitle(image1.getTitle() + "-" + image2.getTitle());
      return blendedImage;
   }
}
